/*
 *      The main control program for lameENCdrop
 *
 *	Copyright (c) 2002 John Edwards
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <windows.h>
#include <shellapi.h>
#include <string.h>
#include <stdio.h>
#include <commctrl.h>
#include <shlobj.h>	/* added 20020121 for Output Directory options */

#include "resource.h"
#include "encthread.h"
#include "encode.h"
#include "misc.h"

#define LOSHORT(l)           ((SHORT)(l))
#define HISHORT(l)           ((SHORT)(((DWORD)(l) >> 16) & 0xFFFF))

#define INI_FILE "lamedropXPd.ini"

#define CREATEFONT(sz) \
	CreateFont((sz), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
		VARIABLE_PITCH | FF_SWISS, "")
   

char          fileName[MAX_PATH];
int           frame = 0;
int           encoding_done = 0;
int           encoding;
int           decoding;
int           totalfiles;
int           numfiles;
int           animate = 0;
int           q_value;
double        file_complete;
HINSTANCE     hinst;
HBITMAP       hbm[8];
HMENU         menu;
HFONT         font2;

extern le_options   opt;

extern SettingsLAME iniSettings;           /* iniSettings holds the parameters for the lamedropXPd-configuration */

static const char *resampleCaption[] =
{
	"48000", "44100", "32000", "24000", "22050", "16000", "11025", "8000", 0
};

static const char *converter[] =
{
	"Best", "Medium", "Fast", 0
};

char config_LAMEoutdir[MAX_PATH]="";

//const int approx_kbps[] = { 0, 64, 80, 96, 128, 144, 160, 176, 192, 224, 256 };
//const int approx_kbps[] = { 0, 64, 92, 112, 128, 140, 160, 176, 200, 224, 240 };      /* My quick test 2005-09-17 */
//const int approx_kbps[] = { 0, 64, 88, 104, 120, 132, 152, 166, 198, 228, 244 };      /* HA test 2005-09-21 */
//const int approx_kbps[] = { 0, 60, 84, 100, 118, 132, 152, 166, 190, 220, 240 };      /* My test 2005-09-22 */

static const char *qualityCaption[] = { "High", "Standard", "Fast", 0 };

static const char *GenreCaption[] =
{
	"A Cappella", 		"Acid Jazz", 		"Acid Punk", 		"Acid",
	"Acoustic", 		"Alternative",  	"AlternRock",	 	"Ambient",
 	"Anime", 		"Avantgarde",		"Ballad",	 	"Bass",
 	"Beat", 		"Bebob", 		"Big Band",		"Black Metal",
 	"Bluegrass", 		"Blues", 		"Booty Bass", 		"BritPop",
	"Cabaret", 		"Celtic", 		"Chamber Music",	"Chanson",
 	"Chorus",		"Christian Gangsta Rap","Christian Rap",	"Christian Rock",
	"Classic Rock",		"Classical",		"Club",			"Club-House",
	"Comedy",		"Contemporary Christian","Country",		"Crossover",
	"Cult",			"Dance Hall",		"Dance",		"Darkwave",
	"Death Metal",		"Disco",		"Dream",		"Drum & Bass",
	"Drum Solo",		"Duet",			"Easy Listening",	"Electronic",
	"Ethnic",		"Eurodance",		"Euro-House",		"Euro-Techno",
	"Fast-Fusion",		"Folk",			"Folk/Rock",		"Folklore",
	"Freestyle",		"Funk",			"Fusion",		"Game",
	"Gangsta",		"Goa",			"Gospel",		"Gothic Rock",
	"Gothic",		"Grunge",		"Hard Rock",		"Hardcore",
	"Heavy Metal",		"Hip-Hop",		"House",		"Humour",
	"Indie",		"Industrial",		"Instrumental Pop",	"Instrumental Rock",
	"Instrumental",		"Jazz",			"Jazz+Funk",		"JPop",
	"Jungle",		"Latin",		"Lo-Fi",		"Meditative",
	"Merengue",		"Metal",		"Musical",		"National Folk",
	"Native American",	"Negerpunk",		"New Age",		"New Wave",
	"Noise",		"Oldies",		"Opera",		"Other",
	"Polka",		"Polsk Punk",		"Pop",			"Pop/Funk",
	"Pop-Folk",		"Porn Groove",		"Power Ballad",		"Pranks",
	"Primus",		"Progressive Rock",	"Psychedelic Rock",	"Psychedelic",
	"Punk Rock",		"Punk",			"R&B",			"Rap",
	"Rave",			"Reggae",		"Retro",		"Revival",
	"Rhythmic Soul",	"Rock & Roll",		"Rock",			"Salsa",
	"Samba",		"Satire",		"Showtunes",		"Ska",
	"Slow Jam",		"Slow Rock",		"Sonata",		"Soul",
	"Sound Clip",		"Soundtrack",		"Southern Rock",	"Space",
	"Speech",		"Swing",		"Symphonic Rock",	"Symphony",
	"Synthpop",		"Tango",		"Techno",		"Techno-Industrial",
	"Terror",		"Thrash Metal",		"Top 40",		"Trailer",
	"Trance",		"Tribal",		"Trip-Hop",		"Vocal",
	0
};

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

INT_PTR CALLBACK EncProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) ;
INT_PTR CALLBACK DirProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) ;
INT_PTR CALLBACK TagProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) ;
INT_PTR CALLBACK AboutProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) ;

/*
 * Next function for Output Directory options
 */

static int CALLBACK BrowseCallbackProc( HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	if (uMsg == BFFM_INITIALIZED) {
		SetWindowText(hwnd,"Select Output Directory");
		SendMessage(hwnd,BFFM_SETSELECTION,(WPARAM)1,(LPARAM)config_LAMEoutdir);
	}
	return 0;
}


/*
 * Write the .ini file using the current lamedropXPd settings
 */
int
WriteIniFile ( const char* Filename )
{
	FILE* fp;

	if ( (fp = fopen (Filename, "w")) == NULL )
		return EOF;                             /* could not open file */

	fprintf (fp, "[lamedropXPd]\n");
	fprintf (fp, "Window_X=%i\n"          , iniSettings.window_x     );
	fprintf (fp, "Window_Y=%i\n"          , iniSettings.window_y     );
	fprintf (fp, "AlwaysOnTop=%i\n"       , iniSettings.always_on_top);
	fprintf (fp, "Logerr=%i\n"            , iniSettings.logerr       );
	fprintf (fp, "DeleteInput=%i\n"       , iniSettings.delete_input );
	fprintf (fp, "SetBitrate=%i\n"        , iniSettings.set_bitrate  );
	fprintf (fp, "Mono=%i\n"              , iniSettings.mono         );
	fprintf (fp, "Quality=%i\n"           , iniSettings.quality      );
	fprintf (fp, "VBRQuality=%i\n"        , iniSettings.vbr_quality  );
	fprintf (fp, "SetCBR=%i\n"            , iniSettings.set_cbr      );
	fprintf (fp, "ABRBitrate=%i\n"        , iniSettings.abr_bitrate  );
	fprintf (fp, "CBRBitrate=%i\n"        , iniSettings.cbr_bitrate  );
	fprintf (fp, "TagMode=%i\n"           , iniSettings.tag_mode     );
	fprintf (fp, "AutoTag=%i\n"           , iniSettings.auto_tag     );
	fprintf (fp, "id3Tag=%i\n"            , iniSettings.id3Tag       );
	fprintf (fp, "Title=%s\n"             , iniSettings.title        );
	fprintf (fp, "Artist=%s\n"            , iniSettings.artist       );
	fprintf (fp, "Album=%s\n"             , iniSettings.album        );
	fprintf (fp, "Genre=%s\n"             , iniSettings.genre        );
	fprintf (fp, "Year=%s\n"              , iniSettings.year         );
	fprintf (fp, "OutputDirMode=%i\n"     , iniSettings.out_dir_mode );
	fprintf (fp, "OutputDir=%s\n"         , iniSettings.out_dir      );
	fprintf (fp, "ScaleMode=%i\n"         , iniSettings.scale_mode   );
	fprintf (fp, "ReSampleRate=%i\n"      , iniSettings.resample_rate);
	fprintf (fp, "ReSampleMode=%i\n"      , iniSettings.resample_mode);
	fprintf (fp, "Converter=%i\n"         , iniSettings.converter    );
	return fclose (fp);
}

/*
 * Read the .ini file and set the lamedropXPd settings
 */
int
ReadIniFile ( FILE* fp )
{
	char  buff [256];
	int   val;

	rewind ( fp );
	fgets ( buff, sizeof buff, fp );

	if ( 0 != memcmp ( buff, "[lamedropXPd]", 13 ) )
		return EOF;

	while ( fgets ( buff, sizeof buff, fp ) != NULL ) {
		if      ( 1 == sscanf ( buff, "Window_X=%d"     , &val ) )  iniSettings.window_x      = val;
		else if ( 1 == sscanf ( buff, "Window_Y=%d"     , &val ) )  iniSettings.window_y      = val;
		else if ( 1 == sscanf ( buff, "Always_on_top=%d", &val ) )  iniSettings.always_on_top = val;
		else if ( 1 == sscanf ( buff, "Logerr=%d"       , &val ) )  iniSettings.logerr        = val;
		else if ( 1 == sscanf ( buff, "DeleteInput=%d"  , &val ) )  iniSettings.delete_input  = val;
		else if ( 1 == sscanf ( buff, "SetBitrate=%d"   , &val ) )  iniSettings.set_bitrate   = val;
		else if ( 1 == sscanf ( buff, "Mono=%d"         , &val ) )  iniSettings.mono          = val;
		else if ( 1 == sscanf ( buff, "Quality=%d"      , &val ) )  iniSettings.quality       = val;
		else if ( 1 == sscanf ( buff, "VBRQuality=%d"   , &val ) )  iniSettings.vbr_quality   = val;
		else if ( 1 == sscanf ( buff, "SetCBR=%d"       , &val ) )  iniSettings.set_cbr       = val;
		else if ( 1 == sscanf ( buff, "ABRBitrate=%d"   , &val ) )  iniSettings.abr_bitrate   = val;
		else if ( 1 == sscanf ( buff, "CBRBitrate=%d"   , &val ) )  iniSettings.cbr_bitrate   = val;
		else if ( 1 == sscanf ( buff, "TagMode=%d"      , &val ) )  iniSettings.tag_mode      = val;
		else if ( 1 == sscanf ( buff, "AutoTag=%d"      , &val ) )  iniSettings.auto_tag      = val;
		else if ( 1 == sscanf ( buff, "id3Tag=%d"       , &val ) )  iniSettings.id3Tag        = val;
		else if ( 1 == sscanf ( buff, "Title=%64[^\r\n]"        , iniSettings.title ) );
		else if ( 1 == sscanf ( buff, "Artist=%64[^\r\n]"       , iniSettings.artist ) );
		else if ( 1 == sscanf ( buff, "Album=%64[^\r\n]"        , iniSettings.album ) );
		else if ( 1 == sscanf ( buff, "Genre=%64[^\r\n]"        , iniSettings.genre ) );
		else if ( 1 == sscanf ( buff, "Year=%32[^\r\n]"         , iniSettings.year  ) );
		else if ( 1 == sscanf ( buff, "OutputDirMode=%d", &val ) )  iniSettings.out_dir_mode  = val;
		else if ( 1 == sscanf ( buff, "OutputDir=%245[^\r\n]"   , iniSettings.out_dir ) );
		else if ( 1 == sscanf ( buff, "ScaleMode=%d"    , &val ) )  iniSettings.scale_mode    = val;
		else if ( 1 == sscanf ( buff, "ReSampleRate=%d" , &val   ) )  iniSettings.resample_rate = val;
		else if ( 1 == sscanf ( buff, "ReSampleMode=%d" , &val   ) )  iniSettings.resample_mode = val;
		else if ( 1 == sscanf ( buff, "Converter=%d"    , &val   ) )  iniSettings.converter     = val;
	}

	return 0;
}


/*
 * Get lamedropXPd settings at startup, writes .ini file, if not present
 */
void
GetlamedropXPdSettings ( void )
{
	FILE*  fp = NULL;
	char   PathAndName [] = {INI_FILE};

	/* set default values */
	iniSettings.window_x      =    64;        /* default box position (x co-ord)                         */
	iniSettings.window_y      =    64;        /* default box position (y co-ord)                         */
	iniSettings.always_on_top =     8;        /* default = on                                            */
	iniSettings.logerr        =     8;        /* default = on                                            */
	iniSettings.delete_input  =     0;        /* default = off                                           */
	iniSettings.set_bitrate   =     0;        /* default = 0, use VBRquality                             */
	iniSettings.mono          =     0;        /* default = 0, 2 channel                                  */
	iniSettings.quality       =     1;        /* default = normal                                        */
	iniSettings.vbr_quality   =   801;        /* default = 801, old --preset standard                     */
	iniSettings.set_cbr       =     0;        /* default = 0, use ABR                                    */
	iniSettings.abr_bitrate   =   128;        /* default = 128kbps                                       */
	iniSettings.cbr_bitrate   =   128;        /* default = 128kbps                                       */
	iniSettings.tag_mode      =     0;        /* default = default                                       */
	iniSettings.auto_tag      =     0;        /* default = off                                           */
	iniSettings.id3Tag        =     0;        /* default = V1 & V2(if necessary)                         */
	strcpy(iniSettings.title, "");            /* default = blank                                         */
	strcpy(iniSettings.artist, "");           /* default = blank                                         */
	strcpy(iniSettings.album, "");            /* default = blank                                         */
	strcpy(iniSettings.genre, "Other");       /* default = Other                                         */
	strcpy(iniSettings.year, "");             /* default = blank                                         */
	iniSettings.out_dir_mode  =    0;         /* default = same as input dir                             */
	strcpy(iniSettings.out_dir, "");          /* default = blank (as above)                              */
	iniSettings.scale_mode    =    0;         /* default = don't use ogg/FLAC scaling (1=Title, 2=Album) */
	iniSettings.resample_rate =44100;        // default = 44100 (samples per second)
	iniSettings.resample_mode =    0;        // default = output rate = input rate
	iniSettings.converter     =    1;        // default = medium quality

	/* Read INI_FILE */
	if ( (fp = fopen (PathAndName, "r")) == NULL ) {    /* file does not exist: write it! */
		WriteIniFile ( PathAndName );
	}
	else {                                              /* file does exist: read it! */
		ReadIniFile (fp);
		fclose (fp);
	}

	return;
}

void set_set_bitrate(int v)
{
	encthread_setset_bitrate(v);
	iniSettings.set_bitrate = v;
}

void set_mono(int v)
{
	encthread_setmono(v);
	iniSettings.mono = v;
}

void set_quality(int v)
{
	iniSettings.quality = v;
	if (v == 0)      v = 2;
	else if (v == 1) v = 3;
	else if (v == 2) v = 7;
	encthread_setquality(v);
}

void set_vbr_quality(int v)
{
	encthread_setvbr_quality(v);
	iniSettings.vbr_quality = v;
	q_value = v;
}

void set_set_cbr(int v)
{
	encthread_setset_cbr(v);
	iniSettings.set_cbr = v;
}

void set_abr_bitrate(int v)
{
	encthread_setabrbitrate(v);
	iniSettings.abr_bitrate = v;
}

void set_cbr_bitrate(int v)
{
	encthread_setcbrbitrate(v);
	iniSettings.cbr_bitrate = v;
}

void set_tag_mode(int v)
{
	encthread_settag_mode(v);
	iniSettings.tag_mode = v;
}

void set_id3(int v)
{
	encthread_setid3(v);
	iniSettings.id3Tag = v;
}

void set_resample_rate(int v)
{
	opt.resamplefreq = v;
	if (v != 0) {
		iniSettings.resample_rate = v;
	} else {
		iniSettings.resample_rate = 44100;
	}
}

void set_resample_mode(int v)
{
	iniSettings.resample_mode = v;
}

void set_converter(int v)
{
	opt.converter = v;
	iniSettings.converter = v;
}

void set_scale_mode(int v)
{
	encthread_setscale_mode(v);
	iniSettings.scale_mode = v;
}

void set_AUTOtagging(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_AUTO_TAG, v ? MF_CHECKED : MF_UNCHECKED);
	encthread_setAUTOtagging(v);
	iniSettings.auto_tag = v;
}

void set_always_on_top(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_ON_TOP, v ? MF_CHECKED : MF_UNCHECKED);
	SetWindowPos(hwnd, v ? HWND_TOPMOST : HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOSIZE | SWP_SHOWWINDOW | SWP_NOMOVE);
	iniSettings.always_on_top = v;
}

void set_logerr(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_ERRORS, v ? MF_CHECKED : MF_UNCHECKED);
	iniSettings.logerr = v;
	set_use_dialogs(v);
}

void set_delInput(HWND hwnd, int v)
{
	CheckMenuItem(menu, IDM_DEL_INPUT, v ? MF_CHECKED : MF_UNCHECKED);
	encthread_setdelInput(v);
	iniSettings.delete_input = v;
}

void set_title(char v[64])
{
	encthread_set_title(v);
	strcpy(iniSettings.title, v);
}

void set_artist(char v[64])
{
	encthread_set_artist(v);
	strcpy(iniSettings.artist, v);
}

void set_album(char v[64])
{
	encthread_set_album(v);
	strcpy(iniSettings.album, v);
}

void set_dates(char v[32])
{
	encthread_set_dates(v);
	strcpy(iniSettings.year, v);
}

void set_genre(char v[64])
{
	encthread_set_genre(v);
	strcpy(iniSettings.genre, v);
}

void set_out_dir_mode(int v)
{
	iniSettings.out_dir_mode = v;
}

void set_out_dir(char *v)
{
	strcpy(iniSettings.out_dir, v);
}

int nearestBitrateFull(const int bitrate)
{
	/* borrowed from DM abr presets and amended to return nearest bitrate */    

	int index; /* resolved range */

	const int bitrate_table[] = {8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320};

	int lower_range_kbps = 0,
	upper_range_kbps = 0;
    
	int b;

	/* We assume specified bitrate will be 320kbps */
	upper_range_kbps = bitrate_table[16];
	lower_range_kbps = bitrate_table[16];
 
	/* Determine which significant bitrates the value specified falls between,
	 * if loop ends without breaking then we were correct above that the value was 320
	 */
	for (b = 0; b < 16; b++) {
		if ((Max(bitrate, bitrate_table[b+1])) != bitrate) {
			upper_range_kbps = bitrate_table[b+1];
			lower_range_kbps = bitrate_table[b];
			break; /* We found upper range */
		}
	}

	/* Determine which range the value specified is closer to */
	if ((upper_range_kbps - bitrate) > (bitrate - lower_range_kbps))
		index = lower_range_kbps;
	else
		index = upper_range_kbps;

	return index;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	static char szAppName[] = "lamedropXPd";
	HWND        hwnd;
	MSG         msg;
	WNDCLASSEX  wndclass;
	const       int width = 130;
	const       int height = 130;
	int         x;
	int         y;

	hinst = hInstance;

	wndclass.cbSize        = sizeof(WNDCLASSEX);
	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc   = WndProc;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = 0;
	wndclass.hInstance     = hInstance;
	wndclass.hIcon         = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
	wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	wndclass.lpszMenuName  = NULL;
	wndclass.lpszClassName = szAppName;
	wndclass.hIconSm       = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON2));

	RegisterClassEx(&wndclass);

	GetlamedropXPdSettings();

	x = max(min(iniSettings.window_x, GetSystemMetrics(SM_CXSCREEN) - width), 0);
	y = max(min(iniSettings.window_y, GetSystemMetrics(SM_CYSCREEN) - height), 0);

	hwnd = CreateWindow(szAppName, "lamedropXPd", WS_POPUP | WS_DLGFRAME, x, y,
			width, height, NULL, NULL, hinst, NULL);

	ShowWindow(hwnd, nCmdShow);
	UpdateWindow(hwnd);

	font2 = CREATEFONT(10);

	SetTimer(hwnd, 1, 80, NULL);

	/*
	 * Apply all the .ini settings
	 */

	set_set_bitrate(iniSettings.set_bitrate);
	set_mono(iniSettings.mono);
	set_quality(iniSettings.quality);
	set_vbr_quality(iniSettings.vbr_quality);
	set_set_cbr(iniSettings.set_cbr);
	set_abr_bitrate(iniSettings.abr_bitrate);
	set_cbr_bitrate(iniSettings.cbr_bitrate);
	set_tag_mode(iniSettings.tag_mode);
	set_resample_mode(iniSettings.resample_mode);
	set_scale_mode(iniSettings.scale_mode);
	set_resample_rate(!iniSettings.resample_mode ? 0 : iniSettings.resample_rate);
	set_converter(iniSettings.converter);
	set_AUTOtagging(hwnd, iniSettings.auto_tag);
	set_id3(iniSettings.id3Tag);
	set_always_on_top(hwnd, iniSettings.always_on_top);
	set_logerr(hwnd, iniSettings.logerr);
	set_delInput(hwnd, iniSettings.delete_input);
	set_title(iniSettings.title);
	set_artist(iniSettings.artist);
	set_album(iniSettings.album);
	set_genre(iniSettings.genre);
	set_dates(iniSettings.year);
	set_out_dir_mode(iniSettings.out_dir_mode);
	if (iniSettings.out_dir_mode) {
		set_out_dir(iniSettings.out_dir);
		strcpy(config_LAMEoutdir, iniSettings.out_dir);
	}
	else {
		set_out_dir("");
		strcpy(config_LAMEoutdir, "");
	}
	
	for (frame = 0; frame < 8; frame++)
		hbm[frame] = LoadImage(hinst, MAKEINTRESOURCE(IDB_BITMAP1 + frame), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);
	frame = 0;

	while (GetMessage(&msg, NULL, 0, 0)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	for (frame = 0; frame < 8; frame++) 
		DeleteObject(hbm[frame]);

	return msg.wParam;
}

void HandleDrag(HWND hwnd, HDROP hDrop)
{
	int    cFiles,
	       i,
	       flag = 0;
	char   szFile[MAX_PATH],
	       *ext;
	HANDLE event = NULL;

	cFiles = DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);
	for (i = 0; i < cFiles; i++) {
		DragQueryFile(hDrop, i, szFile, sizeof(szFile));

		if (ext = strrchr(szFile, '.')) {
			if ((local_strcasecmp(ext, ".wav") == 0 || local_strcasecmp(ext, ".aif") == 0) ||
			    (local_strcasecmp(ext, ".ogg") == 0 || local_strcasecmp(ext, ".flac") == 0)) {
				flag = 1;
				encthread_addfile(szFile);
				encoding = 1;
				decoding = 0;
			}
			else if ((local_strcasecmp(ext, ".mpg") == 0 || local_strcasecmp(ext, ".mp1") == 0) ||
			         (local_strcasecmp(ext, ".mp2") == 0 || local_strcasecmp(ext, ".mp3") == 0)) {
				flag = 1;
				encthread_addfile(szFile);
				encoding = 0;
				decoding = 1;
			}
			else
				MessageBox(NULL, "Unsupported input file type", NULL, MB_OK | MB_ICONEXCLAMATION);
		}
	}

	DragFinish(hDrop);

	if (flag)
		SetEvent(event);
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC          hdc,
	             hmem;
	static HDC   offscreen;
	PAINTSTRUCT  ps;
	RECT         rect;
	BITMAP       bm;
	POINT        point;
	static POINT start;
	static int   dragging = 0;
	HDC          desktop;
	HBITMAP      hbitmap;
	HANDLE       hdrop;
	HFONT        dfltFont;
	int          dfltBGMode;
	double       percomp;
	int 	 	 width;
	int 	 	 height;
	RECT         bar1,
    	         bar2,
        	     vbrBR;

	switch (message) {

		case WM_CREATE:
			menu = LoadMenu(hinst, MAKEINTRESOURCE(IDR_MENU1));
			menu = GetSubMenu(menu, 0);

			offscreen = CreateCompatibleDC(NULL);
			desktop = GetDC(GetDesktopWindow());
			hbitmap = CreateCompatibleBitmap(desktop, 200, 200);
			ReleaseDC(GetDesktopWindow(), desktop);
			SelectObject(offscreen, hbitmap);

			// Start the engines
			encthread_init();

			// We accept drag&drop
			DragAcceptFiles(hwnd, TRUE);
			return 0;

		case WM_PAINT:
			hdc = BeginPaint(hwnd, &ps);
			GetClientRect(hwnd, &rect);
			width = rect.right + 1;
			height = rect.bottom + 1;

			FillRect(offscreen, &rect, (HBRUSH)GetStockObject(WHITE_BRUSH));
			DrawText(offscreen, "Drop Files Here", -1, &rect, DT_SINGLELINE | DT_CENTER);

			hmem = CreateCompatibleDC(offscreen);
			SelectObject(hmem, hbm[frame]);
			GetObject(hbm[frame], sizeof(BITMAP), &bm);
			BitBlt(offscreen, width / 2 - 31, height / 2 - 33, bm.bmWidth, bm.bmHeight, hmem, 0, 0, SRCCOPY);
			DeleteDC(hmem);
		
			percomp = ((double)(totalfiles - numfiles) + 1 - (1 - file_complete)) / (double)totalfiles;

			SetRect(&vbrBR, 0, height - 35, width, height - 19);

			dfltBGMode = SetBkMode(offscreen, TRANSPARENT);
			dfltFont = SelectObject(offscreen, font2);

			SetRect(&bar1, 0, height - 23, (int)(file_complete * width), height - 13);
			SetRect(&bar2, 0, height - 12, (int)(percomp * width), height - 2);

			FillRect(offscreen, &bar1, (HBRUSH)GetStockObject(LTGRAY_BRUSH));
			FillRect(offscreen, &bar2, (HBRUSH)GetStockObject(DKGRAY_BRUSH));

			if (fileName[0]) {
				char  fileCaption[80];

				strcpy(fileCaption, " ");
				strncat(fileCaption, fileName, 78);

				DrawText(offscreen, fileCaption, -1, &bar1, DT_SINGLELINE | DT_LEFT);
			}

			SelectObject(offscreen, dfltFont);
			SetBkMode(offscreen, dfltBGMode);

			BitBlt(hdc, 0, 0, width, height, offscreen, 0, 0, SRCCOPY);

			EndPaint(hwnd, &ps);

			DefWindowProc(hwnd, message, wParam, lParam);
			return 0;

		case WM_TIMER:
			if (animate || frame) {
				frame++;
				if (frame > 7) 
					frame -= 8;
			} else {
				frame = 0;
			}
			GetClientRect(hwnd, &rect);
			InvalidateRect(hwnd, &rect, FALSE);
			return 0;

		case WM_LBUTTONDOWN:
			start.x = LOWORD(lParam);
			start.y = HIWORD(lParam);
			ClientToScreen(hwnd, &start);
			GetWindowRect(hwnd, &rect);
			start.x -= rect.left;
			start.y -= rect.top;
			dragging = 1;
			SetCapture(hwnd);
			return 0;

		case WM_LBUTTONUP:
			if (dragging) {
				dragging = 0;
				ReleaseCapture();
			}
			return 0;

		case WM_MOUSEMOVE:
			if (dragging) {
				point.x = LOSHORT(lParam);
				point.y = HISHORT(lParam);

				/* lParam can contain negative coordinates !
				point.x = LOWORD(lParam);
				point.y = HIWORD(lParam);
				*/

				ClientToScreen(hwnd, &point);
				SetWindowPos(hwnd, 0, point.x - start.x, point.y - start.y, 0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_SHOWWINDOW);
				iniSettings.window_x = point.x - start.x;
				iniSettings.window_y = point.y - start.y;
			}
			return 0;

		case WM_CAPTURECHANGED:
			if (dragging) {
				dragging = 0;
				ReleaseCapture();
			}
			return 0;

		case WM_RBUTTONUP:
			point.x = LOWORD(lParam);
			point.y = HIWORD(lParam);
			ClientToScreen(hwnd, &point);
			TrackPopupMenu(menu, TPM_RIGHTBUTTON, point.x, point.y, 0, hwnd, NULL);
			return 0;

		case WM_COMMAND:
			switch (LOWORD(wParam)) {

 				case IDM_QUIT:
					WriteIniFile(INI_FILE);
					encoding_done = 1;
					PostQuitMessage(0);
					break;
				case IDM_AUTO_TAG:
					set_AUTOtagging(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;
				case IDM_ON_TOP:
					set_always_on_top(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;	
				case IDM_ERRORS:
					set_logerr(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;
				case IDM_DEL_INPUT:
					set_delInput(hwnd, ~GetMenuState(menu, LOWORD(wParam), MF_BYCOMMAND) & MF_CHECKED);
					break;
				case IDM_ENC_OPTS: {
//					int value = DialogBox(hinst, MAKEINTRESOURCE(IDD_ENC), hwnd, EncProc);
					DialogBox(hinst, MAKEINTRESOURCE(IDD_ENC), hwnd, EncProc);

					break;
				}
				case IDM_OUT_DIR: {
//					int value = DialogBox(hinst, MAKEINTRESOURCE(IDD_DIR), hwnd, DirProc);
					INT_PTR value = DialogBox(hinst, MAKEINTRESOURCE(IDD_DIR), hwnd, DirProc);

					if (value == -4) {
						strcpy(config_LAMEoutdir, "");
					}
					break;
				}
				case IDM_TAG_SETUP: {
//					int value = DialogBox(hinst, MAKEINTRESOURCE(IDD_TAG_SETUP), hwnd, TagProc);
					DialogBox(hinst, MAKEINTRESOURCE(IDD_TAG_SETUP), hwnd, TagProc);

					break;
				}
				case IDM_ABOUT: {
//					int value = DialogBox(hinst, MAKEINTRESOURCE(IDD_ABOUT), hwnd, AboutProc);
					DialogBox(hinst, MAKEINTRESOURCE(IDD_ABOUT), hwnd, AboutProc);

					break;
				}
			} // LOWORD(wParam)
			return 0;

		case WM_DROPFILES:
			hdrop = (HANDLE)wParam;
			HandleDrag(hwnd, hdrop);
			return 0;
		
		case WM_DESTROY:
			encoding_done = 1;
			PostQuitMessage(0);
			return 0;

		default:
			return DefWindowProc(hwnd, message, wParam, lParam);
	}
	return 0;
}

 /*
  *  Encode parameters dialog procedures.
  */
INT_PTR CALLBACK EncProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
	char       editBuf[128];
	char       editBuf2[128];
	int        br;
	int        quality_value;
	int        cbr_bitrate;
	int        abr_bitrate;
	int        approx_kbps;
	float      VBR_Q;
	static int use_cbr = 0;

	switch (message) { 
		case WM_INITDIALOG: {

			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			if(iniSettings.set_bitrate == 1) {
				CheckDlgButton(hwndDlg, IDC_BITRATE, TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_SLIDER_QUALITY), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_V_VALUE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_QUALITY_VALUES), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_APPROX_BITRATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_TARGET_VBR_MODE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_VBR_MODE), FALSE);
			}
			else {
				CheckDlgButton(hwndDlg, IDC_QUALITY, TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_QUALITY_SETTING), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_TARGET_BITRATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_BITRATE_KBPS), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_BITRATE_SELECTED), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CBR_ENC), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CBR_TEXT), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CBR_BITRATE), FALSE);
			}

			for (br=0; resampleCaption[br] != 0; br++) {
				SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) resampleCaption[br]);

				SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) atoi(resampleCaption[br]));
			}

			/* set to default sample rate */
			for (br=0; resampleCaption[br] != 0; br++) {
				if ( atoi(resampleCaption[br]) == iniSettings.resample_rate) {
					SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_SETCURSEL,
	        	  			(WPARAM) br, (LPARAM) 0);
					break;
				}
			}

			if (iniSettings.resample_mode == 0) {
				CheckDlgButton(hwndDlg,IDC_RESAMPLE_NEW, FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_RATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK1), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CONVERTER), FALSE);
			}
			else if (iniSettings.resample_mode == 1) {
				CheckDlgButton(hwndDlg,IDC_RESAMPLE_NEW, TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_RATE), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK1), TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CONVERTER), TRUE);
			}

			for (br=0; converter[br] != 0; br++) {
				SendDlgItemMessage(hwndDlg, IDC_CONVERTER, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) converter[br]);

				SendDlgItemMessage(hwndDlg, IDC_CONVERTER, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) converter[br]);
			}

			SendDlgItemMessage(hwndDlg, IDC_CONVERTER, CB_SETCURSEL,
	            		(WPARAM) iniSettings.converter, (LPARAM) 0);

		   	SendDlgItemMessage(hwndDlg, IDC_BITRATE_SELECTED, EM_SETLIMITTEXT,
        			(WPARAM) 4, (LPARAM)0);

			abr_bitrate = iniSettings.abr_bitrate;
			sprintf(editBuf, "%d", abr_bitrate);
			SendDlgItemMessage(hwndDlg, IDC_BITRATE_SELECTED, WM_SETTEXT,(WPARAM)0, (LPARAM)editBuf);

			SendDlgItemMessage(hwndDlg, IDC_TARGET_BITRATE, WM_SETTEXT,(WPARAM)0,
			    (LPARAM)"Enter Target Bitrate:");
			SendDlgItemMessage(hwndDlg, IDC_BITRATE_KBPS, WM_SETTEXT,(WPARAM)0, (LPARAM)"kbps");
			SendDlgItemMessage(hwndDlg, IDC_CBR_TEXT, WM_SETTEXT,(WPARAM)0,
			    (LPARAM)"Restrict encoder to Constant Bitrate (CBR)");
			SendDlgItemMessage(hwndDlg, IDC_TARGET_VBR_MODE, WM_SETTEXT,(WPARAM)0,
			    (LPARAM)"Variable Bitrate Mode:");
			SendDlgItemMessage(hwndDlg, IDC_QUALITY_VALUES, WM_SETTEXT,(WPARAM)0,
//			   (LPARAM)"10       20       30       40       50       60       70       80       90      100");
			   (LPARAM)"0       10      20      30      40      50      60      70      80      90      100");
// Quality Slider
			SendDlgItemMessage(hwndDlg, IDC_SLIDER_QUALITY, TBM_SETRANGE, 
        			(WPARAM) TRUE,                   // redraw flag 
        			(LPARAM) MAKELONG(1, 1001));  // min. & max. positions 

			SendDlgItemMessage(hwndDlg, IDC_SLIDER_QUALITY, TBM_SETPAGESIZE, 
        			0, (LPARAM) 4);                  // new page size 
 
		   	SendDlgItemMessage(hwndDlg, IDC_SLIDER_QUALITY, TBM_SETSEL, 
        			(WPARAM) FALSE,                  // redraw flag 
		        	(LPARAM) MAKELONG(1, 1001));

		   	SendDlgItemMessage(hwndDlg, IDC_SLIDER_QUALITY, TBM_SETPOS, 
        			(WPARAM) TRUE,                   // redraw flag 
		        	(LPARAM) (q_value));

		   	SendDlgItemMessage(hwndDlg, IDC_APPROX_BITRATE, EM_SETLIMITTEXT,
        			(WPARAM) 25, (LPARAM)0);

			approx_kbps = q_value * 192 / 1001 + 64;
			sprintf(editBuf, "Approx. Bitrate %d kbps", approx_kbps);
			SendDlgItemMessage(hwndDlg, IDC_APPROX_BITRATE, WM_SETTEXT,(WPARAM)0, (LPARAM)editBuf);

			VBR_Q = ((q_value - 1001) * -1.f) / 100;
			if (VBR_Q == 0.00)
				VBR_Q *= -1.f;
			sprintf(editBuf2, "%-4.2f", VBR_Q);
			SendDlgItemMessage(hwndDlg, IDC_V_VALUE, WM_SETTEXT,(WPARAM)0, (LPARAM)editBuf2);

			if(iniSettings.mono == 1)
				CheckDlgButton(hwndDlg, IDC_MONO_ENC, TRUE);

			cbr_bitrate = iniSettings.cbr_bitrate;
			if(iniSettings.set_cbr == 1) {
				use_cbr = 1;
				CheckDlgButton(hwndDlg, IDC_CBR_ENC, TRUE);
				if(iniSettings.set_bitrate == 1)
					EnableWindow(GetDlgItem(hwndDlg, IDC_CBR_BITRATE), TRUE);
				sprintf(editBuf, "CBR encoding will use a bitrate of %d kbps. L.A.M.E. will automatically resample input, where necessary, for low bitrates.", cbr_bitrate);
				SendDlgItemMessage(hwndDlg, IDC_CBR_BITRATE, WM_SETTEXT,(WPARAM)0, (LPARAM)editBuf);
			}
			else {
				sprintf(editBuf, "");
				SendDlgItemMessage(hwndDlg, IDC_CBR_BITRATE, WM_SETTEXT,(WPARAM)0, (LPARAM)editBuf);
			}

			for (br=0; qualityCaption[br] != 0; br++) {
				SendDlgItemMessage(hwndDlg, IDC_QUALITY_SETTING, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) qualityCaption[br]);

				SendDlgItemMessage(hwndDlg, IDC_QUALITY_SETTING, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) qualityCaption[br]);
			}

			SendDlgItemMessage(hwndDlg, IDC_QUALITY_SETTING, CB_SETCURSEL,
			        (WPARAM) iniSettings.quality, (LPARAM) 0);

			if (iniSettings.scale_mode == 1) {
				CheckDlgButton(hwndDlg,IDC_OGGFLAC_TITLE, TRUE);
			}
			else if (iniSettings.scale_mode == 2) {
				CheckDlgButton(hwndDlg,IDC_OGGFLAC_ALBUM, TRUE);
			}

			return TRUE;
		}
		case WM_HSCROLL: {

			quality_value = SendDlgItemMessage(hwndDlg, IDC_SLIDER_QUALITY, TBM_GETPOS,
							  (WPARAM)0, (LPARAM)0);

			q_value = quality_value;

			approx_kbps = q_value * 192 / 1001 + 64;

			sprintf(editBuf, "Approx. Bitrate %d kbps", approx_kbps);
			SendDlgItemMessage(hwndDlg, IDC_APPROX_BITRATE, WM_SETTEXT,(WPARAM)0, (LPARAM)editBuf);

			VBR_Q = ((q_value - 1001) * -1.f) / 100;
			if (VBR_Q == 0.00)
				VBR_Q *= -1.f;

			sprintf(editBuf2, "%-4.2f", VBR_Q);
			SendDlgItemMessage(hwndDlg, IDC_V_VALUE, WM_SETTEXT,(WPARAM)0, (LPARAM)editBuf2);

			return TRUE;
		}
		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 
			switch (LOWORD(wParam)) {
				case IDCANCEL:

					EndDialog(hwndDlg, FALSE);
					break;

				case IDC_BUTTON1: {

					int    resample_rate;

					if ((IsDlgButtonChecked(hwndDlg, IDC_CHECK1) == BST_CHECKED) &&
					    (IsDlgButtonChecked(hwndDlg, IDC_RESAMPLE_NEW) == BST_CHECKED)) {
						if ((br = SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
							set_resample_rate(SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE,
									CB_GETITEMDATA, (WPARAM) br, (LPARAM) 0));
							resample_rate = SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE,
									CB_GETITEMDATA, (WPARAM) br, (LPARAM) 0);
							set_resample_mode(0);
						}
					}
					else if (IsDlgButtonChecked(hwndDlg, IDC_RESAMPLE_NEW) == BST_CHECKED) {
						if ((br = SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, CB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
							set_resample_rate(SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE, 
									CB_GETITEMDATA,	(WPARAM) br, (LPARAM) 0));
							resample_rate = SendDlgItemMessage(hwndDlg, IDC_RESAMPLE_RATE,
									CB_GETITEMDATA, (WPARAM) br, (LPARAM) 0);
							set_resample_mode(1);
						}
					}
					else {
						set_resample_mode(0);
						set_resample_rate(0);
						resample_rate = 44100;
					}

					if ((br = SendDlgItemMessage(hwndDlg, IDC_CONVERTER, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0)) != CB_ERR) {
						set_converter(br);
					}

					set_vbr_quality(q_value);

					if (IsDlgButtonChecked(hwndDlg, IDC_BITRATE) == BST_CHECKED)
						set_set_bitrate(1);
					else
						set_set_bitrate(0);

					if (IsDlgButtonChecked(hwndDlg, IDC_MONO_ENC) == BST_CHECKED)
						set_mono(1);
					else
						set_mono(0);

					if (IsDlgButtonChecked(hwndDlg, IDC_CBR_ENC) == BST_CHECKED)
						set_set_cbr(1);
					else
						set_set_cbr(0);

					set_quality((int)SendDlgItemMessage(hwndDlg, IDC_QUALITY_SETTING, CB_GETCURSEL,
							(WPARAM) 0, (LPARAM) 0));

					GetDlgItemText(hwndDlg, IDC_BITRATE_SELECTED, editBuf, sizeof(editBuf));
					br = atoi(editBuf);
					if (br < 8)
						br = 8;
					cbr_bitrate = nearestBitrateFull(br);
					set_cbr_bitrate(cbr_bitrate);
					set_abr_bitrate(br);

					if (IsDlgButtonChecked(hwndDlg, IDC_OGGFLAC_TITLE) == BST_CHECKED) {
						set_scale_mode(1);
						set_tag_mode(2);
					}
					else if (IsDlgButtonChecked(hwndDlg, IDC_OGGFLAC_ALBUM) == BST_CHECKED) {
						set_scale_mode(2);
						set_tag_mode(2);
					}
					else
						set_scale_mode(0);

//					EndDialog(hwndDlg, TRUE);
					EndDialog(hwndDlg, FALSE);

//					return TRUE;
				}
				break;
				
				case IDC_OGGFLAC_TITLE:

					if (IsDlgButtonChecked(hwndDlg, IDC_OGGFLAC_TITLE) == BST_CHECKED) {
						CheckDlgButton(hwndDlg,IDC_OGGFLAC_ALBUM,FALSE);
					}
					break;

				case IDC_OGGFLAC_ALBUM:

					if (IsDlgButtonChecked(hwndDlg, IDC_OGGFLAC_ALBUM) == BST_CHECKED) {
						CheckDlgButton(hwndDlg,IDC_OGGFLAC_TITLE,FALSE);
					}
					break;

				case IDC_QUALITY: {
					CheckDlgButton(hwndDlg, IDC_BITRATE, FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_V_VALUE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_QUALITY_VALUES), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SLIDER_QUALITY), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_APPROX_BITRATE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_TARGET_VBR_MODE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_VBR_MODE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_QUALITY_SETTING), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_TARGET_BITRATE), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_BITRATE_KBPS), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_BITRATE_SELECTED), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_CBR_ENC), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_CBR_TEXT), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_CBR_BITRATE), FALSE);
					break;
				}

				case IDC_BITRATE: {
					CheckDlgButton(hwndDlg, IDC_QUALITY, FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_V_VALUE), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SLIDER_QUALITY), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_QUALITY_VALUES), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_APPROX_BITRATE), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_TARGET_VBR_MODE), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_VBR_MODE), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_QUALITY_SETTING), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_TARGET_BITRATE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_BITRATE_KBPS), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_BITRATE_SELECTED), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_CBR_ENC), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_CBR_TEXT), TRUE);
					if (IsDlgButtonChecked(hwndDlg, IDC_CBR_ENC) == BST_CHECKED)
						EnableWindow(GetDlgItem(hwndDlg, IDC_CBR_BITRATE), TRUE);
					break;
				}

				case IDC_CBR_ENC:

					if (IsDlgButtonChecked(hwndDlg, IDC_CBR_ENC) == BST_CHECKED) {
						use_cbr = 1;
						EnableWindow(GetDlgItem(hwndDlg, IDC_CBR_BITRATE), TRUE);
						GetDlgItemText(hwndDlg, IDC_BITRATE_SELECTED, editBuf, sizeof(editBuf));
						br = atoi(editBuf);
						cbr_bitrate = nearestBitrateFull(br);
						sprintf(editBuf, "CBR encoding will use a bitrate of %d kbps. L.A.M.E. will automatically resample input, where necessary, for low bitrates.", cbr_bitrate);
						SendDlgItemMessage(hwndDlg, IDC_CBR_BITRATE, WM_SETTEXT,
							(WPARAM)0, (LPARAM)editBuf);
					}
					else {
						use_cbr = 0;
						EnableWindow(GetDlgItem(hwndDlg, IDC_CBR_BITRATE), FALSE);
						sprintf(editBuf, "");
						SendDlgItemMessage(hwndDlg, IDC_CBR_BITRATE, WM_SETTEXT,
							(WPARAM)0, (LPARAM)editBuf);
					}

				case IDC_V_VALUE:

					switch (HIWORD(wParam)) {
						char buf2[16];

						case EN_UPDATE: {
							int i;
							double v=0.0;

							int len = SendDlgItemMessage(hwndDlg, IDC_V_VALUE, WM_GETTEXT,
								(WPARAM) 6, (LPARAM) buf2);

							for (i=0; i<len; i++) {
								if (! isdigit(buf2[i]) && buf2[i] != '.'
									&& buf2[i] != ',' && buf2[i] != '-') {

									buf2[i] = '\0';
									SendDlgItemMessage(hwndDlg, IDC_V_VALUE, WM_SETTEXT,
										(WPARAM) 0, (LPARAM) buf2);
								}
							}

							v = atof(buf2);

							if ( v > 10.00f ) {
								v = 10.0;
								sprintf(buf2, "%.2f", (float)v);

								SendDlgItemMessage(hwndDlg, IDC_V_VALUE, WM_SETTEXT,
									(WPARAM) 0, (LPARAM) buf2);

								SendDlgItemMessage(hwndDlg, IDC_V_VALUE, EM_SETSEL,
									(WPARAM)0, (LPARAM)-1);
							}
							else if ( v < 0.00f ) {
								v = 0.0;
								sprintf(buf2, "%.2f", (float)v);

								SendDlgItemMessage(hwndDlg, IDC_V_VALUE, WM_SETTEXT,
									(WPARAM) 0, (LPARAM) buf2);

								SendDlgItemMessage(hwndDlg, IDC_V_VALUE, EM_SETSEL,
									(WPARAM)0, (LPARAM)-1);
							}

							break;
						}

						case EN_CHANGE:	{

							double v=0.0;

							SendDlgItemMessage(hwndDlg, IDC_V_VALUE, WM_GETTEXT,
								(WPARAM) 6, (LPARAM) buf2);

							v = atof(buf2);

							q_value = (int)(v * 100.0f * -1.0f + 1001);

							approx_kbps = q_value * 192 / 1001 + 64;

							sprintf(editBuf, "Approx. Bitrate %d kbps", approx_kbps);
							SendDlgItemMessage(hwndDlg, IDC_APPROX_BITRATE, WM_SETTEXT,(WPARAM)0, (LPARAM)editBuf);

						   	SendDlgItemMessage(hwndDlg, IDC_SLIDER_QUALITY, TBM_SETPOS, 
        							(WPARAM) TRUE,                   // redraw flag 
						        	(LPARAM) (q_value));

							break;
						}

						default:
							break;
        	  			}
					break;

				case IDC_BITRATE_SELECTED:
					switch (HIWORD(wParam)) {
						char buf[5];
						int len;
						int i;

						case EN_UPDATE: {

							len = SendDlgItemMessage(hwndDlg, IDC_BITRATE_SELECTED, WM_GETTEXT,
    	          						(WPARAM) 4, (LPARAM) buf);

							buf[len] = '\0';

							for (i=0; i<len; i++) {
								if (!isdigit(buf[i])) {
									buf[i] = '\0';
									SendDlgItemMessage(hwndDlg, IDC_BITRATE_SELECTED, WM_SETTEXT,
										(WPARAM) 0, (LPARAM) buf);
								}
							}
							if (atoi(buf) > 320) {
								SendDlgItemMessage(hwndDlg, IDC_BITRATE_SELECTED, WM_SETTEXT,
									(WPARAM) 0, (LPARAM) "320");

								SendDlgItemMessage(hwndDlg, IDC_BITRATE_SELECTED, EM_SETSEL,
									(WPARAM)0, (LPARAM)-1);
							}
							break;
						}
						case EN_CHANGE: {

							len = SendDlgItemMessage(hwndDlg, IDC_BITRATE_SELECTED, WM_GETTEXT,
    	          						(WPARAM) 4, (LPARAM) buf);

							buf[len] = '\0';

							abr_bitrate = (atoi(buf));
							cbr_bitrate = nearestBitrateFull(abr_bitrate);
							if (use_cbr == 1) {
								sprintf(editBuf, "CBR encoding will use a bitrate of %d kbps. L.A.M.E. will automatically resample input, where necessary, for low bitrates.", cbr_bitrate);
								SendDlgItemMessage(hwndDlg, IDC_CBR_BITRATE, WM_SETTEXT,
									(WPARAM)0, (LPARAM)editBuf);
							}
						}
						break;
//        	  		}
					default:
						break;
        	 	}
				case IDC_RESAMPLE_NEW:

					if (IsDlgButtonChecked(hwndDlg, IDC_RESAMPLE_NEW) == BST_CHECKED) {
						EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_RATE), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK1), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_CONVERTER), TRUE);
					}
					else {
						EnableWindow(GetDlgItem(hwndDlg, IDC_RESAMPLE_RATE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_CHECK1), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_CONVERTER), FALSE);
					}
					break;

				default:
					break;
			}
			break;
	}
return FALSE; 
}

/*
 *  Directory parameters dialog procedures.
 */

INT_PTR CALLBACK DirProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		case WM_INITDIALOG: {

			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			if (iniSettings.out_dir_mode == 1) {
				CheckDlgButton(hwndDlg,IDC_USEOTHERDEF,TRUE);
				SetDlgItemText(hwndDlg, IDC_EDITDIR, config_LAMEoutdir);
			}
			else {
				CheckDlgButton(hwndDlg,IDC_USESAMEDIR,TRUE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_SEL_DIR), FALSE);
				SetDlgItemText(hwndDlg, IDC_EDITDIR, "");
			}
		}
		return TRUE;

		case WM_CLOSE:

			/* Closing the Dialog behaves the same as Cancel */
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break;

		case WM_COMMAND: 

			switch (LOWORD(wParam)) {
				case IDC_BUTTON4: {

					char     szTemp_2[MAX_PATH];
					unsigned int i = 0;
					unsigned int j = 0;
					char     sub[MAX_PATH];

					if (IsDlgButtonChecked(hwndDlg, IDC_USESAMEDIR) == BST_CHECKED) {
						strcpy(config_LAMEoutdir, "");
						set_out_dir("");
						set_out_dir_mode(0);
					}
					else {
						GetDlgItemText(hwndDlg, IDC_EDITDIR, szTemp_2, sizeof(szTemp_2));
						if (!szTemp_2[0]) {
							set_out_dir_mode(0);
							strcpy(config_LAMEoutdir, "");
						}
						else {
							j = strlen(szTemp_2) - 1;
							if (szTemp_2[j] != '\\') {
								szTemp_2[j+1] = '\\';
								szTemp_2[j+2] = '\0';
							}
							strcpy(config_LAMEoutdir, szTemp_2);
						}
						set_out_dir_mode(IsDlgButtonChecked(hwndDlg, IDC_USEOTHERDEF) == BST_CHECKED ? 1 : 0);
						set_out_dir(config_LAMEoutdir);
					}

					EndDialog(hwndDlg, FALSE);
				}
				break;

				case IDCANCEL:

					EndDialog(hwndDlg, FALSE);
					break;

				case IDC_USESAMEDIR:

					SetDlgItemText(hwndDlg, IDC_EDITDIR, "");
					EnableWindow(GetDlgItem(hwndDlg, IDC_SEL_DIR), FALSE);
					break;
									
				case IDC_USEOTHERDIR:

					EnableWindow(GetDlgItem(hwndDlg, IDC_SEL_DIR), TRUE);
					break;

				case IDC_USEOTHERDEF:

					EnableWindow(GetDlgItem(hwndDlg, IDC_SEL_DIR), TRUE);
					break;

									
				case IDC_SEL_DIR: {
					char        szTemp[MAX_PATH];
					char        name[MAX_PATH];
					BROWSEINFO  bi;
					ITEMIDLIST  *idlist;

					bi.hwndOwner = hwndDlg;
					bi.pidlRoot = 0;
					bi.pszDisplayName = name;
					bi.lpszTitle = "Select a directory for output:";
					bi.ulFlags = BIF_NEWDIALOGSTYLE;
					bi.lpfn = BrowseCallbackProc;
					bi.lParam = 0;
					bi.iImage = 0;
					OleInitialize(NULL);
					idlist = SHBrowseForFolder( &bi );

					if (idlist) 
						SHGetPathFromIDList( idlist, szTemp );
					else
						strcpy(szTemp,"");

					SetDlgItemText(hwndDlg, IDC_EDITDIR, szTemp);
				}
				break;

				default:
					break;
			}
			break;
	}
return FALSE; 
}

INT_PTR CALLBACK TagProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
	int br;
	char szTemp[64];

	switch (message) { 
		case WM_INITDIALOG: {
			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);

			if (iniSettings.scale_mode > 0) {
				CheckDlgButton(hwndDlg,IDC_CUST_OPTS,FALSE);
				CheckDlgButton(hwndDlg,IDC_DEF_OPTS,FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CUST_OPTS), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_DEF_OPTS), FALSE);
			}

			if (iniSettings.auto_tag == 8) {
				CheckDlgButton(hwndDlg,IDC_AUTO_TAGGING,TRUE);
				if(iniSettings.tag_mode == 1) {
					CheckDlgButton(hwndDlg, IDC_CUST_OPTS, TRUE);
					CheckDlgButton(hwndDlg,IDC_COPY_COMMENTS,FALSE);
					CheckDlgButton(hwndDlg,IDC_DEF_OPTS,FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_DATE), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), TRUE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), TRUE);
				}
				else {
					CheckDlgButton(hwndDlg, IDC_DEF_OPTS, TRUE);
					CheckDlgButton(hwndDlg,IDC_COPY_COMMENTS,FALSE);
					CheckDlgButton(hwndDlg,IDC_CUST_OPTS,FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_DATE), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), FALSE);
					EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), FALSE);
				}
			}
			else {
				CheckDlgButton(hwndDlg,IDC_AUTO_TAGGING,FALSE);
				if(iniSettings.tag_mode == 2) {
					CheckDlgButton(hwndDlg,IDC_COPY_COMMENTS,TRUE);
				}
				else {
					CheckDlgButton(hwndDlg,IDC_COPY_COMMENTS,FALSE);
				}
				CheckDlgButton(hwndDlg,IDC_CUST_OPTS,FALSE);
				CheckDlgButton(hwndDlg,IDC_DEF_OPTS,FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_DEF_OPTS), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_CUST_OPTS), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_DATE), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), FALSE);
				EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), FALSE);
			}

			SendDlgItemMessage(hwndDlg, IDC_ARTIST, WM_SETTEXT,(WPARAM)0, (LPARAM)iniSettings.artist);
			SendDlgItemMessage(hwndDlg, IDC_ALBUM, WM_SETTEXT,(WPARAM)0, (LPARAM)iniSettings.album);
			SendDlgItemMessage(hwndDlg, IDC_TITLE, WM_SETTEXT,(WPARAM)0, (LPARAM)iniSettings.title);
			SendDlgItemMessage(hwndDlg, IDC_DATE, WM_SETTEXT,(WPARAM)0, (LPARAM)iniSettings.year);

			for (br=0; GenreCaption[br] != 0; br++) {
				SendDlgItemMessage(hwndDlg, IDC_GENRE, CB_ADDSTRING,
	        	  		(WPARAM)0, (LPARAM) GenreCaption[br]);

				SendDlgItemMessage(hwndDlg, IDC_GENRE, CB_SETITEMDATA, 
	        	  		(WPARAM) br, (LPARAM) GenreCaption[br]);
			}

			for(br=0; GenreCaption[br] != 0; br++) {
				if (!strcmp(GenreCaption[br], iniSettings.genre)) {
        				SendDlgItemMessage(hwndDlg, IDC_GENRE, CB_SETCURSEL,
			            		(WPARAM) br, (LPARAM) 0);
        				break;
				}
			}

			if(iniSettings.id3Tag == 0)
				CheckDlgButton(hwndDlg, IDC_ID3V1, TRUE);
			else if(iniSettings.id3Tag == 1)
				CheckDlgButton(hwndDlg, IDC_ID3V2, TRUE);
			else if(iniSettings.id3Tag == 2)
				CheckDlgButton(hwndDlg, IDC_ID3V1_ONLY, TRUE);
			else if(iniSettings.id3Tag == 3)
				CheckDlgButton(hwndDlg, IDC_ID3V2_ONLY, TRUE);

			return TRUE;
		}

		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 
			switch (LOWORD(wParam)) {
				case IDCANCEL:

					EndDialog(hwndDlg, FALSE);
					break;

				case IDC_ACCEPT_TAG:
					if (IsDlgButtonChecked(hwndDlg, IDC_AUTO_TAGGING) == BST_CHECKED) {
						set_AUTOtagging(hwndDlg, 8);
						if (IsDlgButtonChecked(hwndDlg, IDC_DEF_OPTS) == BST_CHECKED) {
							set_tag_mode(0);
						}
						else if (IsDlgButtonChecked(hwndDlg, IDC_CUST_OPTS) == BST_CHECKED) {
							set_tag_mode(1);
							if ((br = SendDlgItemMessage(hwndDlg, IDC_GENRE, CB_GETCURSEL,
								(WPARAM) 0, (LPARAM) 0)) != CB_ERR)
								strcpy(szTemp, GenreCaption[br]);
							else
								GetDlgItemText(hwndDlg, IDC_GENRE, szTemp, sizeof(szTemp));
							if(strlen(szTemp) > 0)
								set_genre(szTemp);
							else
								set_genre("");
							GetDlgItemText(hwndDlg, IDC_DATE, szTemp, sizeof(szTemp));
							set_dates(szTemp);
							GetDlgItemText(hwndDlg, IDC_TITLE, szTemp, sizeof(szTemp));
							set_title(szTemp);
							GetDlgItemText(hwndDlg, IDC_ARTIST, szTemp, sizeof(szTemp));
							set_artist(szTemp);
							GetDlgItemText(hwndDlg, IDC_ALBUM, szTemp, sizeof(szTemp));
							set_album(szTemp);
						}
					}
					else if (IsDlgButtonChecked(hwndDlg, IDC_COPY_COMMENTS) == BST_CHECKED) {
						set_AUTOtagging(hwndDlg, 0);
						set_tag_mode(2);
					}
					else {
						set_AUTOtagging(hwndDlg, 0);
						set_tag_mode(0);
					}

					if (IsDlgButtonChecked(hwndDlg, IDC_ID3V1) == BST_CHECKED)
						set_id3(0);
					else if (IsDlgButtonChecked(hwndDlg, IDC_ID3V2) == BST_CHECKED)
						set_id3(1);
					else if (IsDlgButtonChecked(hwndDlg, IDC_ID3V1_ONLY) == BST_CHECKED)
						set_id3(2);
					else if (IsDlgButtonChecked(hwndDlg, IDC_ID3V2_ONLY) == BST_CHECKED)
						set_id3(3);

					EndDialog(hwndDlg, TRUE);
					return FALSE;

				case IDC_COPY_COMMENTS:
					if (IsDlgButtonChecked(hwndDlg, IDC_COPY_COMMENTS) == BST_CHECKED) {
						CheckDlgButton(hwndDlg,IDC_AUTO_TAGGING,FALSE);
						CheckDlgButton(hwndDlg,IDC_DEF_OPTS,FALSE);
						CheckDlgButton(hwndDlg,IDC_CUST_OPTS,FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_DEF_OPTS), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_CUST_OPTS), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_DATE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), FALSE);
					}
					break;

				case IDC_AUTO_TAGGING:
					if (IsDlgButtonChecked(hwndDlg, IDC_AUTO_TAGGING) == BST_CHECKED) {
						EnableWindow(GetDlgItem(hwndDlg, IDC_DEF_OPTS), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_CUST_OPTS), TRUE);
						if(iniSettings.tag_mode == 1) {
							CheckDlgButton(hwndDlg, IDC_CUST_OPTS, TRUE);
							CheckDlgButton(hwndDlg,IDC_COPY_COMMENTS,FALSE);
							CheckDlgButton(hwndDlg,IDC_DEF_OPTS,FALSE);
							EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), TRUE);
							EnableWindow(GetDlgItem(hwndDlg, IDC_DATE), TRUE);
							EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), TRUE);
							EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), TRUE);
							EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), TRUE);
						}
						else {
							CheckDlgButton(hwndDlg, IDC_DEF_OPTS, TRUE);
							CheckDlgButton(hwndDlg,IDC_COPY_COMMENTS,FALSE);
							CheckDlgButton(hwndDlg,IDC_CUST_OPTS,FALSE);
							EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), FALSE);
							EnableWindow(GetDlgItem(hwndDlg, IDC_DATE), FALSE);
							EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), FALSE);
							EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), FALSE);
							EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), FALSE);
						}
					}
					else {
						CheckDlgButton(hwndDlg, IDC_DEF_OPTS, FALSE);
						CheckDlgButton(hwndDlg,IDC_COPY_COMMENTS,FALSE);
						CheckDlgButton(hwndDlg,IDC_CUST_OPTS,FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_DATE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), FALSE);
					}
					break;

				case IDC_DEF_OPTS:
					if (IsDlgButtonChecked(hwndDlg, IDC_DEF_OPTS) == BST_CHECKED) {
						CheckDlgButton(hwndDlg,IDC_COPY_COMMENTS,FALSE);
						CheckDlgButton(hwndDlg,IDC_CUST_OPTS,FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_DATE), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), FALSE);
					}
					break;

				case IDC_CUST_OPTS:
					if (IsDlgButtonChecked(hwndDlg, IDC_CUST_OPTS) == BST_CHECKED) {
						CheckDlgButton(hwndDlg,IDC_DEF_OPTS,FALSE);
						CheckDlgButton(hwndDlg,IDC_COPY_COMMENTS,FALSE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_GENRE), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_DATE), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ARTIST), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_ALBUM), TRUE);
						EnableWindow(GetDlgItem(hwndDlg, IDC_TITLE), TRUE);
					}
					break;

				default:
					break;						
        	 	}
	}
return FALSE; 
}

INT_PTR CALLBACK AboutProc(HWND hwndDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{

	switch (message) { 
		case WM_INITDIALOG: {
			HICON hBgIcon = LoadIcon(hinst, MAKEINTRESOURCE(IDI_ICON1));
			PostMessage(hwndDlg, WM_SETICON, ICON_BIG, (LPARAM)hBgIcon);
#ifdef _WIN64
			SetDlgItemText(hwndDlg, IDC_COMPILE_DATE, "(64 bit)Compiled: " __DATE__);
#else
			SetDlgItemText(hwndDlg, IDC_COMPILE_DATE, "(32 bit)Compiled: " __DATE__);
#endif
						return TRUE;
		}

		case WM_CLOSE:

			// Closing the Dialog behaves the same as Cancel               
			PostMessage(hwndDlg, WM_COMMAND, IDCANCEL, 0);
			break; // End of WM_CLOSE                                      

		case WM_COMMAND: 
			switch (LOWORD(wParam)) {
				case IDCANCEL:

					EndDialog(hwndDlg, FALSE);
					break;

        	 	}
	}
return FALSE; 
}
/*
 * end of main.c
 */



